# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: biointerface.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x62iointerface.proto\x12\x06\x61llego\x1a\x0c\x63ommon.proto\"!\n\x0cSpikesStdReq\x12\x11\n\tdsourceID\x18\x01 \x02(\t\"\x14\n\x12SpikesNewLoadReply\"<\n\x0fSpikesRenameReq\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x16\n\x0eisForceReplace\x18\x02 \x01(\x08\"\x13\n\x11SpikesGetIDsReply\"\x80\x01\n\x14NeuronColorGroupData\x12\x1b\n\x13\x63olorGroupIdxNeuron\x18\x01 \x02(\x05\x12\x1c\n\x14\x63olorGroupIdxChannel\x18\x02 \x02(\x05\x12\x15\n\rposColorGroup\x18\x03 \x02(\x05\x12\x16\n\x0esizeColorGroup\x18\x04 \x02(\x05\"\xb8\x01\n\x17SpikeTrainAnalyticsData\x12\x12\n\nbinnedData\x18\x01 \x03(\x05\x12\x12\n\nbinCenters\x18\x02 \x03(\x01\x12\x0e\n\x06ntvIdx\x18\x03 \x02(\x05\x12\r\n\x05label\x18\x04 \x02(\t\x12\n\n\x02id\x18\x05 \x02(\t\x12\x0e\n\x06yRange\x18\x06 \x03(\x05\x12:\n\x14neuronColorGroupData\x18\x07 \x02(\x0b\x32\x1c.allego.NeuronColorGroupData\"y\n!SpikesGetSpikeTrainAnalyticsReply\x12@\n\x17spikeTrainAnalyticsData\x18\x01 \x03(\x0b\x32\x1f.allego.SpikeTrainAnalyticsData\x12\x12\n\nnumNeurons\x18\x02 \x02(\x05\"\xb2\x02\n#SpikesGetSpikeTrainAnalyticsRequest\x12\x15\n\rspikeSorterID\x18\x01 \x02(\t\x12\x11\n\ttimeRange\x18\x02 \x03(\x01\x12\x10\n\x08\x62inRange\x18\x03 \x03(\x01\x12\x0f\n\x07numBins\x18\x04 \x02(\x03\x12,\n\x08\x61nalytic\x18\x05 \x02(\x0e\x32\x1a.allego.SpikeTrainAnalytic\x12\x14\n\x0c\x63learHistory\x18\x06 \x01(\x08\x12\x16\n\x0eneuronStartIdx\x18\x07 \x01(\x03\x12$\n\x06sortBy\x18\x08 \x02(\x0e\x32\x14.allego.NeuronSortBy\x12(\n\x08sortMode\x18\t \x02(\x0e\x32\x16.allego.NeuronSortMode\x12\x12\n\nmaxNeurons\x18\n \x01(\x03\"\x87\x05\n\x0fSpikesSpecReply\x12\x16\n\x0e\x62iointerfaceID\x18\x01 \x02(\t\x12\x12\n\ndatasetUID\x18\x02 \x02(\t\x12\x10\n\x08\x63hecksum\x18\x03 \x02(\t\x12\x15\n\rprovenanceUID\x18\x04 \x03(\t\x12\x12\n\npersistDur\x18\x06 \x02(\x01\x12\x10\n\x08sampFreq\x18\x07 \x02(\x01\x12\x10\n\x08numSites\x18\x08 \x02(\x03\x12\x11\n\tsizeBytes\x18\t \x02(\x03\x12\x1d\n\x15\x65nabledSiteNtvChanIdx\x18\n \x03(\x05\x12\x11\n\ttimeRange\x18\x0b \x03(\x01\x12\x16\n\x0etimestampRange\x18\x0c \x03(\x03\x12\x15\n\rwallTimeStart\x18\r \x02(\t\x12$\n\x04site\x18\x0e \x03(\x0b\x32\x16.allego.SpikesSiteSpec\x12*\n\tsiteStats\x18\x11 \x01(\x0b\x32\x17.allego.SpikesSiteStats\x12\x31\n\tsensorExt\x18\x12 \x01(\x0b\x32\x1e.allego.SensorExtendedMetadata\x12(\n\x07project\x18\x13 \x01(\x0b\x32\x17.allego.ProjectMetadata\x12)\n\x07\x62ioType\x18\x14 \x02(\x0e\x32\x18.allego.BiointerfaceType\x12\x17\n\x0freqNumNeighbors\x18\x15 \x01(\x03\x12\x1c\n\x14neighborhoodRadiusUm\x18\x16 \x01(\x01\x12\x1d\n\x02tR\x18\x17 \x01(\x0b\x32\x11.allego.TimeRange\x12\x1f\n\x02sG\x18\x18 \x01(\x0b\x32\x13.allego.SignalGroup\x12\"\n\x07spikeTR\x18\x19 \x01(\x0b\x32\x11.allego.TimeRange\"\xf7\x04\n\x0eSpikesSiteSpec\x12\x12\n\nntvChanIdx\x18\x01 \x02(\x05\x12,\n\x08neighbor\x18\x02 \x02(\x0b\x32\x1a.allego.NeighborDescriptor\x12\x11\n\tisEnabled\x18\x03 \x02(\x08\x12\x10\n\x08posProbe\x18\x04 \x03(\x01\x12\x11\n\tposTissue\x18\x05 \x03(\x01\x12\x10\n\x08sampFreq\x18\x06 \x02(\x01\x12\x15\n\rwallTimeStart\x18\x07 \x02(\t\x12\x11\n\ttimeRange\x18\x08 \x03(\x01\x12\x16\n\x0etimestampRange\x18\t \x03(\x03\x12\x16\n\x0espikeTimeRange\x18\n \x03(\x01\x12\x1b\n\x13spikeTimestampRange\x18\x0b \x03(\x03\x12\x1c\n\x14spikeTimeRangeMemory\x18\x0c \x03(\x01\x12!\n\x19spikeTimestampRangeMemory\x18\r \x03(\x03\x12\x1c\n\x14spikeCountSiteMemory\x18\x0e \x01(\x03\x12\x16\n\x0espikeCountSite\x18\x0f \x03(\x01\x12\x15\n\rspikeRateSite\x18\x10 \x03(\x01\x12\x14\n\x0cnoiseLevelSd\x18\x11 \x02(\x01\x12\x0f\n\x07snrSite\x18\x12 \x02(\x01\x12(\n\x06neuron\x18\x13 \x03(\x0b\x32\x18.allego.NeuronDescriptor\x12\x12\n\nnumNeurons\x18\x14 \x01(\x05\x12,\n\x07\x63ompLen\x18\x15 \x02(\x0b\x32\x1b.allego.SpikeComponentShape\x12\x1d\n\x02tR\x18\x16 \x01(\x0b\x32\x11.allego.TimeRange\x12\"\n\x07spikeTR\x18\x17 \x01(\x0b\x32\x11.allego.TimeRange\"!\n\x10SpikeLabelRecord\x12\r\n\x05label\x18\x01 \x03(\x05\"\x9a\x02\n\x0bSpikeMatrix\x12\x11\n\tnumSpikes\x18\x01 \x02(\x03\x12\x17\n\x0ftimestampsBytes\x18\x02 \x02(\x0c\x12\x0e\n\x06labels\x18\x03 \x03(\t\x12\x15\n\rwaveformBytes\x18\x04 \x01(\x0c\x12\x13\n\x0bwaveformLen\x18\x05 \x01(\x05\x12\x15\n\rfeaturesBytes\x18\x06 \x01(\x0c\x12\x13\n\x0b\x66\x65\x61turesLen\x18\x07 \x01(\x05\x12\x16\n\x0e\x66\x65\x61tures2Bytes\x18\x08 \x01(\x0c\x12\x14\n\x0c\x66\x65\x61tures2Len\x18\t \x01(\x05\x12\x13\n\x0bzscoreBytes\x18\n \x01(\x0c\x12\x11\n\tzscoreLen\x18\x0b \x01(\x05\x12\x0f\n\x07siteIdx\x18\x0c \x01(\x05\x12\x10\n\x08spikePos\x18\r \x03(\x05\"}\n\x0eSpikeMatrixSet\x12\x0b\n\x03uID\x18\x01 \x02(\t\x12\x0f\n\x07siteIdx\x18\x02 \x03(\x05\x12\x16\n\x0etimestampRange\x18\x03 \x03(\x03\x12\x10\n\x08sampFreq\x18\x04 \x02(\x01\x12#\n\x06spkMtx\x18\x05 \x03(\x0b\x32\x13.allego.SpikeMatrix\"\x93\x01\n\x14NeuronSpikeMatrixSet\x12\x0e\n\x06labels\x18\x01 \x03(\t\x12\x10\n\x08sampFreq\x18\x02 \x02(\x01\x12\x0f\n\x07siteIdx\x18\x03 \x03(\x05\x12#\n\x06spkMtx\x18\x04 \x03(\x0b\x32\x13.allego.SpikeMatrix\x12\x16\n\x0etimestampRange\x18\x05 \x03(\x03\x12\x0b\n\x03uID\x18\x06 \x02(\t\"\xa1\x02\n\x16SpikesGetSpikesRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\'\n\x04mode\x18\x02 \x02(\x0e\x32\x19.allego.NeuronsSignalMode\x12\x12\n\nntvChanIdx\x18\x03 \x03(\x05\x12\x11\n\ttimeStart\x18\x04 \x02(\x01\x12\x10\n\x08timeStop\x18\x05 \x02(\x01\x12-\n\x0bspikeLabels\x18\x06 \x03(\x0b\x32\x18.allego.SpikeLabelRecord\x12\x1b\n\x13maxSpikesPerChannel\x18\x07 \x02(\x05\x12\x33\n\tspikeComp\x18\x08 \x03(\x0e\x32 .allego.SpikeDescriptorComponent\x12\x11\n\tspkSortID\x18\t \x01(\t\"\x99\x01\n\x1a\x42iointerfaceGetHistRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12%\n\x04mode\x18\x02 \x02(\x0e\x32\x17.allego.HistScalingMode\x12\x12\n\nntvChanIdx\x18\x03 \x03(\x05\x12-\n\x0bspikeLabels\x18\x04 \x03(\x0b\x32\x18.allego.SpikeLabelRecord\"\x90\x01\n%BiointerfaceGetSpikeTimestampsRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x12\n\nntvChanIdx\x18\x02 \x03(\x05\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12-\n\x0bspikeLabels\x18\x06 \x03(\x0b\x32\x18.allego.SpikeLabelRecord\"\x8b\x01\n BiointerfaceSpikeTimestampsReply\x12<\n\x18spikeTimestampsByChannel\x18\x02 \x03(\x0b\x32\x1a.allego.SpikeTimestampData\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12\x16\n\x0etimeStampRange\x18\x04 \x03(\x03\"S\n)BiointerfaceSeekEndSpikeTimestampsRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x13\n\x0b\x63omponentID\x18\x02 \x02(\t\"Y\n\x1d\x42iointerfaceGetNeuronsRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x12\n\nntvChanIdx\x18\x02 \x03(\x05\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\"\xa1\x03\n\x1b\x42iointerfaceGetNeuronsReply\x12;\n\x04site\x18\x01 \x03(\x0b\x32-.allego.BiointerfaceGetNeuronsReply.SiteEntry\x1a\x36\n\x0eSpikeRateChunk\x12\x11\n\ttimeRange\x18\x01 \x03(\x01\x12\x11\n\tspikeRate\x18\x02 \x03(\x01\x1a\xb2\x01\n\x07NeuronL\x12(\n\x06neuron\x18\x01 \x03(\x0b\x32\x18.allego.NeuronDescriptor\x12:\n\x14neuronColorGroupData\x18\x02 \x03(\x0b\x32\x1c.allego.NeuronColorGroupData\x12\x41\n\x05\x63hunk\x18\x03 \x01(\x0b\x32\x32.allego.BiointerfaceGetNeuronsReply.SpikeRateChunk\x1aX\n\tSiteEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12:\n\x05value\x18\x02 \x01(\x0b\x32+.allego.BiointerfaceGetNeuronsReply.NeuronL:\x02\x38\x01\"\x87\x02\n\x0fSpikesSiteStats\x12\x17\n\x0fspikeCountStats\x18\t \x03(\x01\x12\x1e\n\x16labeledSpikeCountStats\x18\n \x03(\x01\x12 \n\x18unlabeledSpikeCountStats\x18\x0b \x03(\x01\x12\x16\n\x0espikeRateStats\x18\x0c \x03(\x01\x12\x1d\n\x15labeledSpikeRateStats\x18\r \x03(\x01\x12\x1f\n\x17unlabeledSpikeRateStats\x18\x0e \x03(\x01\x12\x16\n\x0esiteNoiseStats\x18\x0f \x03(\x01\x12\x14\n\x0csiteSnrStats\x18\x10 \x03(\x01\x12\x13\n\x0bneuronStats\x18\x11 \x03(\x01\"W\n\x13SpikeComponentShape\x12\x10\n\x08waveform\x18\x01 \x03(\x05\x12\x0f\n\x07\x66\x65\x61ture\x18\x02 \x03(\x05\x12\x10\n\x08\x66\x65\x61ture2\x18\x03 \x03(\x05\x12\x0b\n\x03pos\x18\x04 \x03(\x05\"\xa8\x01\n\x12NeighborDescriptor\x12\x0e\n\x06radius\x18\x01 \x02(\x01\x12\x12\n\nntvChanIdx\x18\x02 \x03(\x05\x12\x10\n\x08\x64istance\x18\x03 \x03(\x01\x12\x10\n\x08thetaDeg\x18\x04 \x03(\x01\x12\x0e\n\x06phiDeg\x18\x05 \x03(\x01\x12\x0b\n\x03num\x18\x06 \x02(\x05\x12\x14\n\x0cSiteCenterUm\x18\x07 \x03(\x01\x12\x17\n\x0fSiteCenterTcsUm\x18\x08 \x03(\x01\"\xf3\x01\n\x10NeuronDescriptor\x12\x10\n\x08neuronID\x18\x01 \x02(\t\x12\x16\n\x0esiteNtvChanIdx\x18\x02 \x02(\x05\x12\x12\n\nspikeLabel\x18\x03 \x02(\t\x12\x10\n\x08posProbe\x18\x04 \x03(\x01\x12\x11\n\tposTissue\x18\x05 \x03(\x01\x12\x12\n\nspikeCount\x18\x06 \x02(\x03\x12\x11\n\tspikeRate\x18\x07 \x02(\x01\x12\x16\n\x0emeanAbsPeakWfm\x18\x08 \x01(\x01\x12\x0b\n\x03snr\x18\t \x01(\x01\x12\x30\n\x08metaData\x18\n \x01(\x0b\x32\x1e.allego.NeuronExtendedMetadata\"\x86\x01\n\x16NeuronExtendedMetadata\x12\x18\n\x10neuronCatalogUID\x18\x01 \x01(\t\x12\x13\n\x0b\x65nsembleUID\x18\x02 \x01(\t\x12\x17\n\x0f\x66uncAssemblyUID\x18\x03 \x03(\t\x12\x12\n\ndatasetUID\x18\x04 \x01(\t\x12\x10\n\x08probeUID\x18\x05 \x01(\t\"\x8b\x01\n\x10NeuronStatistics\x12\x1a\n\x12WaveformAvgAbsAmpl\x18\x01 \x02(\x01\x12\x0b\n\x03Snr\x18\x02 \x02(\x01\x12\x12\n\nspikeCount\x18\x03 \x02(\x03\x12\x14\n\x0c\x61vgSpikeRate\x18\x04 \x02(\x01\x12$\n\x04isih\x18\x05 \x01(\x0b\x32\x16.allego.NeuronHistData\"|\n\x0fSpikeDescriptor\x12\x11\n\ttimestamp\x18\x01 \x02(\x03\x12\r\n\x05label\x18\x02 \x02(\t\x12\x10\n\x08waveform\x18\x03 \x03(\x02\x12\x10\n\x08\x66\x65\x61tures\x18\x04 \x03(\x02\x12\x13\n\x0b\x64\x65\x63\x46\x65\x61tures\x18\x05 \x03(\x01\x12\x0e\n\x06zscore\x18\x06 \x03(\x02\"g\n\tSpikeData\x12&\n\x05spike\x18\x01 \x03(\x0b\x32\x17.allego.SpikeDescriptor\x12\x18\n\x10waveformMaxValue\x18\x02 \x02(\x02\x12\x18\n\x10waveformMinValue\x18\x03 \x02(\x02\"1\n\rSpikeTimeData\x12\x11\n\tspikeTime\x18\x01 \x03(\x01\x12\r\n\x05label\x18\x02 \x03(\x05\"V\n\x12SpikeTimestampData\x12\x17\n\x0fntvChanIdxArray\x18\x01 \x03(\x05\x12\x17\n\x0fspikeTimestamps\x18\x02 \x03(\x03\x12\x0e\n\x06labels\x18\x03 \x03(\x05\"\x85\x01\n\x0eSpikeTimestamp\x12\x16\n\x0espikeTimestamp\x18\x01 \x02(\x03\x12:\n\x14neuronColorGroupData\x18\x02 \x02(\x0b\x32\x1c.allego.NeuronColorGroupData\x12\r\n\x05label\x18\x03 \x02(\x05\x12\x10\n\x08\x64\x61taIdxs\x18\x04 \x03(\x05\"\xb6\x02\n\x17SpikeTemplateDescriptor\x12\x10\n\x08neuronID\x18\x01 \x02(\t\x12\r\n\x05label\x18\x02 \x02(\t\x12\x11\n\tnumSpikes\x18\x03 \x02(\x03\x12\x10\n\x08waveform\x18\x04 \x02(\x0c\x12\x12\n\nwaveformSd\x18\x05 \x02(\x0c\x12\x13\n\x0blenWaveform\x18\x06 \x02(\x03\x12\x10\n\x08\x66\x65\x61tures\x18\x07 \x02(\x0c\x12\x12\n\nfeaturesSd\x18\x08 \x02(\x0c\x12\x13\n\x0blenFeatures\x18\t \x02(\x03\x12\x11\n\tfeatures2\x18\n \x02(\x0c\x12\x13\n\x0b\x66\x65\x61tures2Sd\x18\x0b \x02(\x0c\x12\x14\n\x0clenFeatures2\x18\x0c \x02(\x03\x12\x0e\n\x06zscore\x18\r \x02(\x0c\x12\x10\n\x08zscoreSd\x18\x0e \x02(\x0c\x12\x11\n\tlenZscore\x18\x0f \x02(\x03\"[\n\x12SiteSpikeTemplates\x12\x12\n\nntvChanIdx\x18\x01 \x02(\x05\x12\x31\n\x08template\x18\x02 \x03(\x0b\x32\x1f.allego.SpikeTemplateDescriptor\"\x97\x01\n\x1f\x42iointerfaceSpikeTemplatesReply\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x10\n\x08sampFreq\x18\x02 \x02(\x01\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12\x12\n\nntvChanIdx\x18\x04 \x03(\x05\x12(\n\x04site\x18\x05 \x03(\x0b\x32\x1a.allego.SiteSpikeTemplates\"@\n\x14\x42iointerfaceMetaData\x12(\n\x0bsignalGroup\x18\x01 \x01(\x0b\x32\x13.allego.SignalGroup\"\xd6\x01\n\x1a\x42iointerfaceSpikeDataReply\x12\x16\n\x0e\x62iointerfaceID\x18\x01 \x02(\t\x12\x10\n\x08sampFreq\x18\x02 \x02(\x01\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12\x16\n\x0etimestampRange\x18\x04 \x03(\x03\x12\x12\n\nntvChanIdx\x18\x05 \x03(\x05\x12\x1f\n\x04site\x18\x06 \x03(\x0b\x32\x11.allego.SpikeData\x12.\n\x08metaData\x18\x07 \x01(\x0b\x32\x1c.allego.BiointerfaceMetaData\"\xb5\x01\n\x1fSpikeDataDenseChannelDescriptor\x12\x12\n\nntvChanIdx\x18\x01 \x02(\x05\x12\x0e\n\x06labels\x18\x02 \x03(\x05\x12:\n\x14neuronColorGroupData\x18\x03 \x03(\x0b\x32\x1c.allego.NeuronColorGroupData\x12\x18\n\x10waveformMaxValue\x18\x04 \x02(\x02\x12\x18\n\x10waveformMinValue\x18\x05 \x02(\x02\"\xb0\x01\n\x19SpikesSpikeDataDenseReply\x12\x11\n\ttimeRange\x18\x01 \x03(\x01\x12\x17\n\x0fwaveformNPoints\x18\x02 \x02(\x05\x12\x14\n\x0ctotalNSpikes\x18\x03 \x02(\x05\x12\x43\n\x12\x63hannelDescriptors\x18\x04 \x03(\x0b\x32\'.allego.SpikeDataDenseChannelDescriptor\x12\x0c\n\x04\x64\x61ta\x18\x05 \x02(\x0c\"\x8a\x01\n\x17GroupSpikeTimeDataReply\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x10\n\x08sampFreq\x18\x02 \x02(\x01\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12\x12\n\nntvChanIdx\x18\x04 \x03(\x05\x12#\n\x04site\x18\x05 \x03(\x0b\x32\x15.allego.SpikeTimeData\"v\n\x16\x42iointerfaceSpiketrain\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x11\n\ttimeRange\x18\x02 \x03(\x01\x12\x10\n\x08sampFreq\x18\x03 \x02(\x01\x12$\n\x04site\x18\x04 \x03(\x0b\x32\x16.allego.SiteSpiketrain\"p\n\x0eSiteSpiketrain\x12\x12\n\nntvChanIdx\x18\x01 \x02(\x05\x12\"\n\x06neuron\x18\x02 \x03(\x0b\x32\x12.allego.Spiketrain\x12\x10\n\x08sampFreq\x18\x03 \x02(\x01\x12\x14\n\x0cnumNeighbors\x18\x04 \x02(\x05\"\x8b\x02\n\nSpiketrain\x12\x10\n\x08sampFreq\x18\x01 \x02(\x01\x12\r\n\x05label\x18\x02 \x02(\x05\x12\x17\n\x0fspikeTimestamps\x18\x03 \x03(\x03\x12\x14\n\x0cwaveformData\x18\x05 \x03(\x02\x12\x15\n\rwaveformShape\x18\x06 \x03(\x05\x12\x13\n\x0b\x66\x65\x61tureData\x18\x07 \x03(\x02\x12\x14\n\x0c\x66\x65\x61tureShape\x18\x08 \x03(\x05\x12\x12\n\nzscoreData\x18\t \x03(\x02\x12\x13\n\x0bzscoreShape\x18\n \x03(\x05\x12,\n\nneuronDesc\x18\x0f \x02(\x0b\x32\x18.allego.NeuronDescriptor\x12\x14\n\x0cnumNeighbors\x18\x11 \x02(\x05\"\xb6\x01\n\x0eNeuronHistData\x12\x13\n\x0b\x62inWidthSec\x18\x01 \x02(\x01\x12\x15\n\rhistTimeRange\x18\x02 \x03(\x01\x12%\n\x04mode\x18\x03 \x02(\x0e\x32\x17.allego.HistScalingMode\x12\x14\n\x0cnumRefEvents\x18\x04 \x02(\x03\x12\x11\n\tnumSpikes\x18\x05 \x02(\x03\x12\x1a\n\x12numProcessedSpikes\x18\x06 \x02(\x03\x12\x0c\n\x04hist\x18\x07 \x03(\x01\"\x9d\x01\n\x0cSiteHistData\x12\x14\n\x0csitePosProbe\x18\x01 \x03(\x01\x12\x14\n\x0csitePosBrain\x18\x02 \x03(\x01\x12(\n\x06neuron\x18\x03 \x03(\x0b\x32\x18.allego.NeuronDescriptor\x12$\n\x04hist\x18\x04 \x03(\x0b\x32\x16.allego.NeuronHistData\x12\x11\n\ttimeRange\x18\x05 \x03(\x01\"\x8b\x01\n\x19\x42iointerfaceHistDataReply\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x10\n\x08sampFreq\x18\x02 \x02(\x01\x12\x11\n\ttimeRange\x18\x03 \x03(\x01\x12\x12\n\nntvChanIdx\x18\x04 \x03(\x05\x12\"\n\x04site\x18\x05 \x03(\x0b\x32\x14.allego.SiteHistData\"\xb4\x01\n\x17\x42iointerfaceSaveRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x0c\n\x04path\x18\x02 \x02(\t\x12\x10\n\x08\x62\x61seName\x18\x03 \x02(\t\x12\x11\n\ttimeRange\x18\x04 \x03(\x01\x12\x33\n\tspikeComp\x18\x05 \x03(\x0e\x32 .allego.SpikeDescriptorComponent\x12\x1e\n\x16includeNeuronTemplates\x18\x06 \x02(\x08\"\xac\x01\n\x17\x42iointerfaceLoadRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x0c\n\x04path\x18\x02 \x02(\t\x12\x10\n\x08\x62\x61seName\x18\x03 \x02(\t\x12\x13\n\x0buidFragment\x18\x04 \x02(\t\x12\x33\n\tspikeComp\x18\x05 \x03(\x0e\x32 .allego.SpikeDescriptorComponent\x12\x14\n\x0c\x66orceReplace\x18\x06 \x02(\x08\"\x95\x01\n\x19\x42iointerfaceSaveLoadReply\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x0c\n\x04path\x18\x02 \x02(\t\x12\x10\n\x08\x62\x61seName\x18\x03 \x02(\t\x12\x0b\n\x03uid\x18\x04 \x02(\t\x12\x1c\n\x14\x62iointerfaceMetaFile\x18\x05 \x02(\t\x12\x1a\n\x12neuronTemplateFile\x18\x06 \x02(\t\"2\n\x17\x42iointerfaceGetIDsReply\x12\x17\n\x0f\x62iointerfaceIDs\x18\x01 \x03(\t\"\xef\x01\n\x16\x42iointerfaceVizRequest\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12\x35\n\x07vizType\x18\x02 \x02(\x0e\x32$.allego.BiointerfaceVizRequest.VType\x12\x12\n\nnumSamples\x18\x03 \x02(\x05\x12\x14\n\x0cisYAutoScale\x18\x04 \x02(\x08\x12\x0c\n\x04yLim\x18\x05 \x03(\x01\x12\x0e\n\x06\x66igDir\x18\x06 \x01(\t\x12\x12\n\nisAutoOpen\x18\x07 \x01(\x08\"/\n\x05VType\x12\x15\n\x11VIZ_BIO_TEMPLATES\x10\x00\x12\x0f\n\x0bVIZ_BIO_DEV\x10\x01\"2\n\x14\x42iointerfaceVizReply\x12\x1a\n\x12pngPathAndFileName\x18\x01 \x01(\t*T\n\x10\x42iointerfaceType\x12\x13\n\x0f\x42IO_SPIKETRAINS\x10\x00\x12\x14\n\x10\x42IO_SPIKES_CACHE\x10\x01\x12\x15\n\x11\x42IO_SPIKES_STREAM\x10\x02* \n\x12SpikesImportSchema\x12\n\n\x06NI_SIM\x10\x00*@\n\x13\x42iointerfaceCommand\x12\x13\n\x0f\x42IO_CMD_SAVE_ON\x10\x00\x12\x14\n\x10\x42IO_CMD_SAVE_OFF\x10\x01*J\n\x11\x42iointerfaceState\x12\x0f\n\x0b\x42IO_SAVE_ON\x10\x00\x12\x10\n\x0c\x42IO_SAVE_OFF\x10\x01\x12\x12\n\x0e\x42IO_CONFIGURED\x10\x02*\x1d\n\x12SpikeTrainAnalytic\x12\x07\n\x03ISI\x10\x00*C\n\x0cNeuronSortBy\x12\x14\n\x10SB_MONITOR_MATCH\x10\x00\x12\x11\n\rSB_SPIKE_RATE\x10\x01\x12\n\n\x06SB_SNR\x10\x02*5\n\x0eNeuronSortMode\x12\x10\n\x0cSM_ASCENDING\x10\x00\x12\x11\n\rSM_DESCENDING\x10\x01*X\n\x11NeuronsSignalMode\x12\n\n\x06Spikes\x10\x00\x12\x13\n\x0fSpikeTimestamps\x10\x01\x12\x0e\n\nSpikeTimes\x10\x02\x12\x12\n\x0eSpikeWaveforms\x10\x03*q\n\x18SpikeDescriptorComponent\x12\x0e\n\nCTimestamp\x10\x00\x12\n\n\x06\x43Label\x10\x01\x12\r\n\tCWaveform\x10\x02\x12\x0c\n\x08\x43\x46\x65\x61ture\x10\x03\x12\x0f\n\x0b\x43\x44\x65\x63\x46\x65\x61ture\x10\x04\x12\x0b\n\x07\x43Zscore\x10\x05*S\n\x0fHistScalingMode\x12\x0c\n\x08\x41sSpikes\x10\x00\x12\x0f\n\x0b\x41sSpikeRate\x10\x01\x12\n\n\x06\x41sNorm\x10\x02\x12\n\n\x06\x41sProb\x10\x03\x12\t\n\x05\x41sRaw\x10\x04\x42\x15Z\x13internal/radix/grpc')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'biointerface_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\023internal/radix/grpc'
  _BIOINTERFACEGETNEURONSREPLY_SITEENTRY._options = None
  _BIOINTERFACEGETNEURONSREPLY_SITEENTRY._serialized_options = b'8\001'
  _BIOINTERFACETYPE._serialized_start=8899
  _BIOINTERFACETYPE._serialized_end=8983
  _SPIKESIMPORTSCHEMA._serialized_start=8985
  _SPIKESIMPORTSCHEMA._serialized_end=9017
  _BIOINTERFACECOMMAND._serialized_start=9019
  _BIOINTERFACECOMMAND._serialized_end=9083
  _BIOINTERFACESTATE._serialized_start=9085
  _BIOINTERFACESTATE._serialized_end=9159
  _SPIKETRAINANALYTIC._serialized_start=9161
  _SPIKETRAINANALYTIC._serialized_end=9190
  _NEURONSORTBY._serialized_start=9192
  _NEURONSORTBY._serialized_end=9259
  _NEURONSORTMODE._serialized_start=9261
  _NEURONSORTMODE._serialized_end=9314
  _NEURONSSIGNALMODE._serialized_start=9316
  _NEURONSSIGNALMODE._serialized_end=9404
  _SPIKEDESCRIPTORCOMPONENT._serialized_start=9406
  _SPIKEDESCRIPTORCOMPONENT._serialized_end=9519
  _HISTSCALINGMODE._serialized_start=9521
  _HISTSCALINGMODE._serialized_end=9604
  _SPIKESSTDREQ._serialized_start=44
  _SPIKESSTDREQ._serialized_end=77
  _SPIKESNEWLOADREPLY._serialized_start=79
  _SPIKESNEWLOADREPLY._serialized_end=99
  _SPIKESRENAMEREQ._serialized_start=101
  _SPIKESRENAMEREQ._serialized_end=161
  _SPIKESGETIDSREPLY._serialized_start=163
  _SPIKESGETIDSREPLY._serialized_end=182
  _NEURONCOLORGROUPDATA._serialized_start=185
  _NEURONCOLORGROUPDATA._serialized_end=313
  _SPIKETRAINANALYTICSDATA._serialized_start=316
  _SPIKETRAINANALYTICSDATA._serialized_end=500
  _SPIKESGETSPIKETRAINANALYTICSREPLY._serialized_start=502
  _SPIKESGETSPIKETRAINANALYTICSREPLY._serialized_end=623
  _SPIKESGETSPIKETRAINANALYTICSREQUEST._serialized_start=626
  _SPIKESGETSPIKETRAINANALYTICSREQUEST._serialized_end=932
  _SPIKESSPECREPLY._serialized_start=935
  _SPIKESSPECREPLY._serialized_end=1582
  _SPIKESSITESPEC._serialized_start=1585
  _SPIKESSITESPEC._serialized_end=2216
  _SPIKELABELRECORD._serialized_start=2218
  _SPIKELABELRECORD._serialized_end=2251
  _SPIKEMATRIX._serialized_start=2254
  _SPIKEMATRIX._serialized_end=2536
  _SPIKEMATRIXSET._serialized_start=2538
  _SPIKEMATRIXSET._serialized_end=2663
  _NEURONSPIKEMATRIXSET._serialized_start=2666
  _NEURONSPIKEMATRIXSET._serialized_end=2813
  _SPIKESGETSPIKESREQUEST._serialized_start=2816
  _SPIKESGETSPIKESREQUEST._serialized_end=3105
  _BIOINTERFACEGETHISTREQUEST._serialized_start=3108
  _BIOINTERFACEGETHISTREQUEST._serialized_end=3261
  _BIOINTERFACEGETSPIKETIMESTAMPSREQUEST._serialized_start=3264
  _BIOINTERFACEGETSPIKETIMESTAMPSREQUEST._serialized_end=3408
  _BIOINTERFACESPIKETIMESTAMPSREPLY._serialized_start=3411
  _BIOINTERFACESPIKETIMESTAMPSREPLY._serialized_end=3550
  _BIOINTERFACESEEKENDSPIKETIMESTAMPSREQUEST._serialized_start=3552
  _BIOINTERFACESEEKENDSPIKETIMESTAMPSREQUEST._serialized_end=3635
  _BIOINTERFACEGETNEURONSREQUEST._serialized_start=3637
  _BIOINTERFACEGETNEURONSREQUEST._serialized_end=3726
  _BIOINTERFACEGETNEURONSREPLY._serialized_start=3729
  _BIOINTERFACEGETNEURONSREPLY._serialized_end=4146
  _BIOINTERFACEGETNEURONSREPLY_SPIKERATECHUNK._serialized_start=3821
  _BIOINTERFACEGETNEURONSREPLY_SPIKERATECHUNK._serialized_end=3875
  _BIOINTERFACEGETNEURONSREPLY_NEURONL._serialized_start=3878
  _BIOINTERFACEGETNEURONSREPLY_NEURONL._serialized_end=4056
  _BIOINTERFACEGETNEURONSREPLY_SITEENTRY._serialized_start=4058
  _BIOINTERFACEGETNEURONSREPLY_SITEENTRY._serialized_end=4146
  _SPIKESSITESTATS._serialized_start=4149
  _SPIKESSITESTATS._serialized_end=4412
  _SPIKECOMPONENTSHAPE._serialized_start=4414
  _SPIKECOMPONENTSHAPE._serialized_end=4501
  _NEIGHBORDESCRIPTOR._serialized_start=4504
  _NEIGHBORDESCRIPTOR._serialized_end=4672
  _NEURONDESCRIPTOR._serialized_start=4675
  _NEURONDESCRIPTOR._serialized_end=4918
  _NEURONEXTENDEDMETADATA._serialized_start=4921
  _NEURONEXTENDEDMETADATA._serialized_end=5055
  _NEURONSTATISTICS._serialized_start=5058
  _NEURONSTATISTICS._serialized_end=5197
  _SPIKEDESCRIPTOR._serialized_start=5199
  _SPIKEDESCRIPTOR._serialized_end=5323
  _SPIKEDATA._serialized_start=5325
  _SPIKEDATA._serialized_end=5428
  _SPIKETIMEDATA._serialized_start=5430
  _SPIKETIMEDATA._serialized_end=5479
  _SPIKETIMESTAMPDATA._serialized_start=5481
  _SPIKETIMESTAMPDATA._serialized_end=5567
  _SPIKETIMESTAMP._serialized_start=5570
  _SPIKETIMESTAMP._serialized_end=5703
  _SPIKETEMPLATEDESCRIPTOR._serialized_start=5706
  _SPIKETEMPLATEDESCRIPTOR._serialized_end=6016
  _SITESPIKETEMPLATES._serialized_start=6018
  _SITESPIKETEMPLATES._serialized_end=6109
  _BIOINTERFACESPIKETEMPLATESREPLY._serialized_start=6112
  _BIOINTERFACESPIKETEMPLATESREPLY._serialized_end=6263
  _BIOINTERFACEMETADATA._serialized_start=6265
  _BIOINTERFACEMETADATA._serialized_end=6329
  _BIOINTERFACESPIKEDATAREPLY._serialized_start=6332
  _BIOINTERFACESPIKEDATAREPLY._serialized_end=6546
  _SPIKEDATADENSECHANNELDESCRIPTOR._serialized_start=6549
  _SPIKEDATADENSECHANNELDESCRIPTOR._serialized_end=6730
  _SPIKESSPIKEDATADENSEREPLY._serialized_start=6733
  _SPIKESSPIKEDATADENSEREPLY._serialized_end=6909
  _GROUPSPIKETIMEDATAREPLY._serialized_start=6912
  _GROUPSPIKETIMEDATAREPLY._serialized_end=7050
  _BIOINTERFACESPIKETRAIN._serialized_start=7052
  _BIOINTERFACESPIKETRAIN._serialized_end=7170
  _SITESPIKETRAIN._serialized_start=7172
  _SITESPIKETRAIN._serialized_end=7284
  _SPIKETRAIN._serialized_start=7287
  _SPIKETRAIN._serialized_end=7554
  _NEURONHISTDATA._serialized_start=7557
  _NEURONHISTDATA._serialized_end=7739
  _SITEHISTDATA._serialized_start=7742
  _SITEHISTDATA._serialized_end=7899
  _BIOINTERFACEHISTDATAREPLY._serialized_start=7902
  _BIOINTERFACEHISTDATAREPLY._serialized_end=8041
  _BIOINTERFACESAVEREQUEST._serialized_start=8044
  _BIOINTERFACESAVEREQUEST._serialized_end=8224
  _BIOINTERFACELOADREQUEST._serialized_start=8227
  _BIOINTERFACELOADREQUEST._serialized_end=8399
  _BIOINTERFACESAVELOADREPLY._serialized_start=8402
  _BIOINTERFACESAVELOADREPLY._serialized_end=8551
  _BIOINTERFACEGETIDSREPLY._serialized_start=8553
  _BIOINTERFACEGETIDSREPLY._serialized_end=8603
  _BIOINTERFACEVIZREQUEST._serialized_start=8606
  _BIOINTERFACEVIZREQUEST._serialized_end=8845
  _BIOINTERFACEVIZREQUEST_VTYPE._serialized_start=8798
  _BIOINTERFACEVIZREQUEST_VTYPE._serialized_end=8845
  _BIOINTERFACEVIZREPLY._serialized_start=8847
  _BIOINTERFACEVIZREPLY._serialized_end=8897
# @@protoc_insertion_point(module_scope)
