import unittest

import radiens.utils.config as config


class base_utest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_metadata_constructor(self):
        pass