# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: datasource.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x64\x61tasource.proto\x12\x06\x61llego\x1a\x0c\x63ommon.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"<\n\x11\x44\x61taSourceRequest\x12\x11\n\tdsourceID\x18\x01 \x03(\t\x12\x14\n\x0crenameDsrcID\x18\x02 \x01(\t\"\xc7\x01\n\x18\x44\x61taSourceSetSaveRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x10\n\x08\x62\x61seName\x18\x02 \x01(\t\x12\x11\n\tdsourceID\x18\x03 \x01(\t\x12\x0b\n\x03uID\x18\x04 \x01(\t\x12(\n\x08\x66ileType\x18\x05 \x01(\x0e\x32\x16.allego.RadixFileTypes\x12\x0f\n\x07isForce\x18\x06 \x01(\x08\x12\x17\n\x0fisBackgroundKPI\x18\x07 \x01(\x08\x12\x17\n\x0fparentDsourceID\x18\x08 \x01(\t\"%\n\x11\x44\x61taSourceIDReply\x12\x10\n\x08sortedID\x18\x01 \x03(\t\"\x95\x01\n\x16\x44\x61taSourceSetSaveReply\x12\x11\n\tdsourceID\x18\x01 \x02(\t\x12(\n\x06status\x18\x02 \x01(\x0b\x32\x18.allego.DataSourceStatus\x12\r\n\x05niIDs\x18\x03 \x03(\t\x12/\n\x0f\x61ssociatedDsrcs\x18\x04 \x03(\x0b\x32\x16.allego.FileDescriptor\"\xae\x01\n\x13\x44\x61taSourceStatusMap\x12\x39\n\x07\x64source\x18\x01 \x03(\x0b\x32(.allego.DataSourceStatusMap.DsourceEntry\x12\x12\n\norderedIDs\x18\x02 \x03(\t\x1aH\n\x0c\x44sourceEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\'\n\x05value\x18\x02 \x01(\x0b\x32\x18.allego.DataSourceStatus:\x02\x38\x01\"\xa1\x03\n\x1f\x43opyRemoveDataSourceFileRequest\x12\x45\n\x04many\x18\x01 \x01(\x0b\x32\x35.allego.CopyRemoveDataSourceFileRequest.MultipleFilesH\x00\x12>\n\x03one\x18\x02 \x01(\x0b\x32/.allego.CopyRemoveDataSourceFileRequest.OneFileH\x00\x12\x0f\n\x07isForce\x18\x03 \x01(\x08\x12\x12\n\nisValidate\x18\x04 \x01(\x08\x1at\n\rMultipleFiles\x12#\n\x03src\x18\x01 \x03(\x0b\x32\x16.allego.FileDescriptor\x12\x10\n\x08\x64\x65stPath\x18\x02 \x01(\t\x12,\n\x0c\x64\x65stFileType\x18\x03 \x01(\x0e\x32\x16.allego.RadixFileTypes\x1aT\n\x07OneFile\x12#\n\x03src\x18\x01 \x02(\x0b\x32\x16.allego.FileDescriptor\x12$\n\x04\x64\x65st\x18\x02 \x01(\x0b\x32\x16.allego.FileDescriptorB\x06\n\x04\x64\x65sc\"\xf8\x02\n\x15\x44\x61taSourceFileSetStat\x12\x10\n\x08numFiles\x18\x01 \x02(\x05\x12\x1b\n\x13numBytesPrimaryData\x18\x02 \x02(\x03\x12\x18\n\x10numBytesMetaData\x18\x03 \x02(\x03\x12\x10\n\x08numBytes\x18\x04 \x02(\x03\x12\x16\n\x0eisMetadataFile\x18\x05 \x02(\x08\x12-\n\ttimeStamp\x18\x06 \x02(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0bnumChannels\x18\x07 \x02(\x05\x12\x13\n\x0b\x64urationSec\x18\x08 \x02(\x01\x12\x12\n\ndatasetUID\x18\t \x02(\t\x12\x17\n\x0f\x64\x61tasetChecksum\x18\n \x01(\t\x12\x19\n\x11\x64\x61tasetProvenance\x18\x0b \x03(\t\x12\x12\n\nsampleRate\x18\x0c \x02(\x05\x12\x37\n\x0e\x62\x61\x63kgroundProc\x18\r \x01(\x0b\x32\x1f.allego.BackgroundProcessStatus\"\x82\x02\n\rCpRmMvLsReply\x12\x10\n\x08numFiles\x18\x01 \x02(\x05\x12\x12\n\nnumDsource\x18\x02 \x02(\x05\x12\x10\n\x08numBytes\x18\x03 \x02(\x03\x12\x10\n\x08\x64\x65stPath\x18\x04 \x02(\t\x12\x35\n\x07\x64source\x18\x05 \x03(\x0b\x32$.allego.CpRmMvLsReply.DataSourceInfo\x12\x0b\n\x03msg\x18\x06 \x01(\t\x1a\x63\n\x0e\x44\x61taSourceInfo\x12$\n\x04\x64\x65sc\x18\x01 \x02(\x0b\x32\x16.allego.FileDescriptor\x12+\n\x04stat\x18\x02 \x02(\x0b\x32\x1d.allego.DataSourceFileSetStat\"y\n\x17\x42\x61\x63kgroundProcessStatus\x12\x0b\n\x03uID\x18\x01 \x02(\t\x12\x14\n\x0c\x66racComplete\x18\x02 \x01(\x01\x12\x16\n\x0e\x65lapsedTimeSec\x18\x03 \x01(\x01\x12#\n\x1b\x65stimatedCompletionWallTime\x18\x04 \x01(\t\"\x8b\x01\n\x19MoveDataSourceFileRequest\x12#\n\x03src\x18\x01 \x02(\x0b\x32\x16.allego.FileDescriptor\x12$\n\x04\x64\x65st\x18\x02 \x02(\x0b\x32\x16.allego.FileDescriptor\x12\x0f\n\x07isForce\x18\x03 \x01(\x08\x12\x12\n\nisValidate\x18\x04 \x01(\x08\"\xbf\x01\n\x10\x44\x61taSourceParams\x12\x12\n\nsampleFreq\x18\x01 \x02(\x01\x12\x11\n\ttimeRange\x18\x02 \x03(\x01\x12\x10\n\x08\x64uration\x18\x03 \x02(\x01\x12%\n\x04type\x18\x05 \x02(\x0e\x32\x17.allego.SignalGroupType\x12\x35\n\x0e\x64\x61tasourceType\x18\x06 \x01(\x0e\x32\x1d.allego.RadiensDatasourceType\x12\x14\n\x0c\x64\x61tasourceID\x18\x07 \x01(\t\".\n\x10ImpedanceRequest\x12\x1a\n\x04port\x18\x01 \x02(\x0e\x32\x0c.allego.Port\"\x85\x01\n\tImpedance\x12\x34\n\nimpedances\x18\x01 \x03(\x0b\x32 .allego.Impedance.ImpedanceValue\x1a\x42\n\x0eImpedanceValue\x12\x12\n\nntvChanIdx\x18\x01 \x02(\x05\x12\x0c\n\x04zMag\x18\x02 \x02(\x01\x12\x0e\n\x06zPhase\x18\x03 \x02(\x01*0\n\x15RadiensDatasourceType\x12\x07\n\x03\x41NY\x10\x00\x12\x0e\n\nTIMESERIES\x10\x01\x42\x15Z\x13internal/radix/grpc')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'datasource_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\023internal/radix/grpc'
  _DATASOURCESTATUSMAP_DSOURCEENTRY._options = None
  _DATASOURCESTATUSMAP_DSOURCEENTRY._serialized_options = b'8\001'
  _RADIENSDATASOURCETYPE._serialized_start=2410
  _RADIENSDATASOURCETYPE._serialized_end=2458
  _DATASOURCEREQUEST._serialized_start=75
  _DATASOURCEREQUEST._serialized_end=135
  _DATASOURCESETSAVEREQUEST._serialized_start=138
  _DATASOURCESETSAVEREQUEST._serialized_end=337
  _DATASOURCEIDREPLY._serialized_start=339
  _DATASOURCEIDREPLY._serialized_end=376
  _DATASOURCESETSAVEREPLY._serialized_start=379
  _DATASOURCESETSAVEREPLY._serialized_end=528
  _DATASOURCESTATUSMAP._serialized_start=531
  _DATASOURCESTATUSMAP._serialized_end=705
  _DATASOURCESTATUSMAP_DSOURCEENTRY._serialized_start=633
  _DATASOURCESTATUSMAP_DSOURCEENTRY._serialized_end=705
  _COPYREMOVEDATASOURCEFILEREQUEST._serialized_start=708
  _COPYREMOVEDATASOURCEFILEREQUEST._serialized_end=1125
  _COPYREMOVEDATASOURCEFILEREQUEST_MULTIPLEFILES._serialized_start=915
  _COPYREMOVEDATASOURCEFILEREQUEST_MULTIPLEFILES._serialized_end=1031
  _COPYREMOVEDATASOURCEFILEREQUEST_ONEFILE._serialized_start=1033
  _COPYREMOVEDATASOURCEFILEREQUEST_ONEFILE._serialized_end=1117
  _DATASOURCEFILESETSTAT._serialized_start=1128
  _DATASOURCEFILESETSTAT._serialized_end=1504
  _CPRMMVLSREPLY._serialized_start=1507
  _CPRMMVLSREPLY._serialized_end=1765
  _CPRMMVLSREPLY_DATASOURCEINFO._serialized_start=1666
  _CPRMMVLSREPLY_DATASOURCEINFO._serialized_end=1765
  _BACKGROUNDPROCESSSTATUS._serialized_start=1767
  _BACKGROUNDPROCESSSTATUS._serialized_end=1888
  _MOVEDATASOURCEFILEREQUEST._serialized_start=1891
  _MOVEDATASOURCEFILEREQUEST._serialized_end=2030
  _DATASOURCEPARAMS._serialized_start=2033
  _DATASOURCEPARAMS._serialized_end=2224
  _IMPEDANCEREQUEST._serialized_start=2226
  _IMPEDANCEREQUEST._serialized_end=2272
  _IMPEDANCE._serialized_start=2275
  _IMPEDANCE._serialized_end=2408
  _IMPEDANCE_IMPEDANCEVALUE._serialized_start=2342
  _IMPEDANCE_IMPEDANCEVALUE._serialized_end=2408
# @@protoc_insertion_point(module_scope)
